#ifndef __APSSE_H__
#define __APSSE_H__

#ifdef  _APSSE_VALUE_DEFINE_
#define _APSSE_
#define _APSSE_VALUE(v)         =(v)
#else
#define _APSSE_                 extern
#define _APSSE_VALUE(v)
#endif

#define VERSION                "2.3.0"

#ifdef WIN32
#define DATADIR                ""
#define CHARSET                "Shift_JIS"
#endif

#define ENV_DIR                DATADIR"env/"
#define DICT_DIR               DATADIR"dict/"
#define TMP_DIR                DATADIR"tmp/"

#define TMP_FILE               TMP_DIR".apsse.tmp"
#define TMP_FILE2              TMP_DIR".apsse2.tmp"
#define TMP_FILE3              TMP_DIR".apsse3.tmp"
#define TMP_FILE4              TMP_DIR".apsse4.tmp"
#define OUT_TMP_FILE           TMP_DIR".apsse.out.tmp"
#define DIC_INDEX_TMP_FILE     TMP_DIR".dic.index.tmp"

#define IGNORE_FILE            ENV_DIR"ignore.txt"
#define REPLACE_FILE           ENV_DIR"replace.txt"
#define SYMBOL_FILE            ENV_DIR"symbol.txt"
#define DICT_FILE              ENV_DIR"dict.txt"
#define STYLE_SHEET            ENV_DIR"style.css"
#define WEBDICT_FILE           ENV_DIR"webdict.txt"
#define WEBWAV_FILE            ENV_DIR"webwav.txt"
#define LABEL_FILE             ENV_DIR"label.txt"

#define DEF_PSSDB5S_FILE       "apsse_out.pssdb5s"
#define DEF_CSV_FILE           "apsse_out.csv"
#define DEF_HTML_FILE          "apsse_out.html"

#define EDITOR                 "apsse Version " VERSION
#define APSSE_URL              "http://www.h5.dion.ne.jp/%7Ekyoun/apsse.html"
#define NOTE1                  "apsseō쐬W/HTML҂̋Ĕzz邱Ƃ́Cς̗LɊւ炸֎~܂D"
#define NOTE2                  "<address>made by <a href=\""APSSE_URL"\" target=\"_blank\">apsse</a> Version " VERSION"</address>"

#define MAX_CHAR               1024
#define MESSAGE_TIME           500
#define DEFAULT_IG_S_L         2
#define DEFAULT_IG_L_L         30
#define DEFAULT_Q_NUM          20
#define DEFAULT_CUT_LENGTH     40
#define DIC_MAX                32

enum
{ T, F };

enum
{ TEXT, HTML };

enum
{ T_WORD, WORD_IDIOM, IDIOM };

enum
{ CR, LF, CR_LF };

enum
{ PSSDB5S, CSV, HTML_OUT, HTML_OUT2 };

enum
{
  EXIT_OPTION = 1,
  EXIT_INPUT_FILE_OPEN, EXIT_OUTPUT_FILE_OPEN, EXIT_DIC_FILE_OPEN,
  EXIT_DICT_FILE_OPEN,
  EXIT_IGNORE_FILE_OPEN, EXIT_REPLACE_FILE_OPEN, EXIT_SYMBOL_FILE_OPEN,
  EXIT_DIC_CR, EXIT_TMP_FILE_OPEN, EXIT_HTTPC, EXIT_STYLE_SHEET_OPEN,
  EXIT_WEBDICT_FILE_OPEN, EXIT_WEBWAV_FILE_OPEN, EXIT_LABEL_FILE_OPEN
};

typedef struct
{
  char s[MAX_CHAR];
  long int o;
}
WORDS;

typedef struct
{
  char e[MAX_CHAR];
  char j[MAX_CHAR];
}
DICT;

_APSSE_ int main(int argc, char **argv);
_APSSE_ FILE *make_tmpfile(const char *, const char *);
_APSSE_ int en_check(const char *, const char *);
_APSSE_ void make_dic_index(FILE *, long int *, int);
_APSSE_ void load_symbol(char *);
_APSSE_ int crlf_check(FILE *);
_APSSE_ void get_option(int, char **, char *, char *, char[][MAX_CHAR],
    FILE **);
_APSSE_ FILE *make_mid_index(FILE *, const char *, const char *);
_APSSE_ int words_cmp(WORDS *, WORDS *);
_APSSE_ FILE *sort_dictionary(FILE *, const char *, int);
_APSSE_ int dict_cmp(DICT *, DICT *);
_APSSE_ int num_cmp(int *, int *);
_APSSE_ void select_question(FILE *, const char *,
    int, const char *, const char *, const char *, char [][MAX_CHAR]);
_APSSE_ void get_file_title(const char *, char *);
_APSSE_ void get_file_name(const char *, char *);
_APSSE_ void ja2hint(char *, char *);
_APSSE_ int idiomcheck(const char *, const char *);
_APSSE_ char *file2memory(const char *);
_APSSE_ void show_help();
_APSSE_ void dict_check(FILE *[], char[][MAX_CHAR]);
_APSSE_ void yen2slash(const char *, char *);
_APSSE_ void html_header_write(FILE *, const char *, const char *, 
			       char [][MAX_CHAR]);
_APSSE_ void url_change(const char *, char *);
_APSSE_ void question_output(FILE *, const char *, const char *, 
			     const char *, char [][3][MAX_CHAR], 
			     char [][MAX_CHAR], char [][MAX_CHAR],
			     int);

_APSSE_ int input_flag _APSSE_VALUE(F);
_APSSE_ int output_flag _APSSE_VALUE(F);
_APSSE_ int text_flag _APSSE_VALUE(HTML);
_APSSE_ int ignore_flag _APSSE_VALUE(T);
_APSSE_ int search_flag _APSSE_VALUE(T_WORD);
_APSSE_ int ignore_short_length _APSSE_VALUE(DEFAULT_IG_S_L);
_APSSE_ int ignore_long_length _APSSE_VALUE(DEFAULT_IG_L_L);
_APSSE_ int replace_flag _APSSE_VALUE(T);
_APSSE_ int sort_dic_flag _APSSE_VALUE(F);
_APSSE_ int question_num _APSSE_VALUE(DEFAULT_Q_NUM);
_APSSE_ int question_flag _APSSE_VALUE(F);
_APSSE_ int idiom_flag _APSSE_VALUE(F);
_APSSE_ int idiom_only_flag _APSSE_VALUE(F);
_APSSE_ int stdin_flag _APSSE_VALUE(F);
_APSSE_ int stdout_flag _APSSE_VALUE(F);
_APSSE_ int format_flag _APSSE_VALUE(PSSDB5S);
_APSSE_ int download_flag _APSSE_VALUE(F);
_APSSE_ int http_flag _APSSE_VALUE(F);
_APSSE_ int d _APSSE_VALUE(0);
_APSSE_ int cut_flag _APSSE_VALUE(F);
_APSSE_ int cut_length _APSSE_VALUE(DEFAULT_CUT_LENGTH);
_APSSE_ int nodict_flag _APSSE_VALUE(F);

#undef _APSSE_
#undef _APSSE_VALUE
#endif
